package textBookEgs_Chapter3_delegatedController;

import java.awt.Canvas;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.event.MouseMotionListener;

import javax.swing.JComponent;

/**
 * Encapsulation of LineDrawView, taken from:
 * 
 * - Olsen Fig 2.5, p. 19 (called DrawView in this eg)
 * 
 * - Fig 3.18 p. 70
 * 
 * @author mb
 * 
 */
public class LineDrawView extends Canvas implements LineModelChangeListener {
	private LineDrawModel myModel;

	public LineDrawView(LineDrawModel model) {
		this.myModel = model;
		LineAppController myListener = new LineAppController(this, myModel);
		// to listen for event dispatched from the model, install listener on
		// the model
		myModel.addListener(this);
		// to listen for mouse motion events, install listener as below
		this.addMouseMotionListener(myListener);
		// to listen for mouse events (which are different that *mouse motion*
		// events, install listener as below
		this.addMouseListener(myListener);
	}

	/*
	 * Paint when the AWT tells us to...
	 */
	public void paint(Graphics g) {
		Graphics2D g2 = (Graphics2D) g;
		for (int i = 0; i < myModel.getNumberOfLines(); i++) {
			g2.drawLine(myModel.getEndPoint1(i).x, myModel.getEndPoint1(i).y,
					myModel.getEndPoint2(i).x, myModel.getEndPoint2(i).y);
		}
	}

	public Dimension getPreferredSize() {
		return new Dimension(400, 200);
	}

	@Override
	public void modelHasChanged() {
		this.invalidate();
	}
	
	@Override
	public void lineWillChange(int index, int newX1, int newY1, int newX2,
			int newY2) {
		Line line = myModel.getLine(index);
		if (line != null) {
			Rectangle oldRegion = new Rectangle(line.getStartPoint().x,
					line.getStartPoint().y, line.getEndPoint().x,
					line.getEndPoint().y);
			this.invalidate(); // need to find a way to damage a region
								// selectively
		}
		Rectangle newRegion = new Rectangle(newX1, newY1, newX2, newY2);
		this.invalidate();// need to find a way to damage a region
		// selectively

	}


}
